import React, { useEffect, useMemo, useRef, useState } from 'react';
import Taro from '@tarojs/taro';
import { View } from '@tarojs/components';
import usePrevious from '@/utils/hooks/usePrevious';
import Column from './Column';
import { useList } from './hooks';
import { classNames, getRect } from './utils';

import './style.less'

/**
 * 核心原理, 左右2栏，一个个放元素，同时记录高度，优先放入低高度区域
 * 元素高度获取，主要是通过图片的 onLoad 来确保加载完成
 * 
 * 复杂度, 多数据公用 list,
 * 需要区分是否更换数据源，是否未更换数据源，只是新增了数据
 */

export default (props) => {
  const { className, listData, itemKey, render } = props

  // rfTimes 用来区分是否更换数据源
  const { list, rfTimes: refresh } = listData

  // 用来绘制页面的数据
  const renderListRef = useRef([])

  const lastList = usePrevious(list)
  const lastRf = usePrevious(refresh)

  // 记录左右2栏的数据及高度
  const leftHeight = useRef(0)
  const rightHeight = useRef(0)
  const [leftList, leftRef, setLeftList] = useList()
  const [rightList, rightRef, setRightList] = useList()

  // 用来强制更新组件
  const [cursor, setCursor] = useState(-1)

  // 每个元素渲染完成, 会拿到当前栏的高度
  const handleRenderFinish = (direct) => (height) => {
    if (direct === 'left') {
      leftHeight.current = height
    } else {
      rightHeight.current = height
    }

    // 触发强制渲染
    setCursor(cursor + 1)
  }
    
  useEffect(() => {
    // 新数据源 = 首次进入 或者 切换数据源
    const isNew = (!lastList?.length && list.length) || (lastRf !== refresh)
    
    if (!isNew) {
      // 非新数据源, list 变更那就说明是执行了上拉加载
      const addedList = []
      for (let item of list) {
        const found = lastList.filter(x => x[itemKey] === item[itemKey])[0]
        if (!found) {
          addedList.push(item)
        }
      }
      
      // 于是, 只继续绘制变更的那部分
      renderListRef.current = addedList
      setCursor(-2)
    } else {
      setCursor(0)
      setLeftList([])
      setRightList([])
      renderListRef.current = list ? list.slice() : []
      leftHeight.current = 0
      rightHeight.current = 0
  
      if (renderListRef.current.length) {
        setCursor(0)
      } else {
        setCursor(-1)
      }
    }
  }, [list])

  useEffect(() => {
    // 如果当前渲染数据为空
    if (!renderListRef.current.length) return;

    // 每次把数组最上面的一个拿出来
    const item = renderListRef.current.shift()
    item.__vid = `f-${Math.random().toString(36).substring(2)}`

    // 优先放入低高度的栏里面
    if (leftHeight.current <= rightHeight.current) {
      setLeftList([...leftRef.current, item])
    } else {
      setRightList([...rightRef.current, item])
    }
  }, [cursor])

  return (
    <View className={classNames(className, 'masonry-layout')}>
      <Column list={leftList} render={render} onRenderFinish={handleRenderFinish('left')} />
      <Column list={rightList} render={render} onRenderFinish={handleRenderFinish('right')} />
    </View>
  )
}
